@@ -2,7 +2,7 @@ package Portage;
 
 # Copyright 1999-2005 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /var/cvsroot/gentoo-src/ufed/Portage.pm,v 1.3 2005/11/13 00:28:17 truedfx Exp $
+# $Header: /var/cvsroot/gentoo-src/ufed/Portage.pm,v 1.2 2005/08/05 13:22:39 truedfx Exp $
 
 my %environment;
 $environment{$_}={} for qw(USE); # INCREMENTALS, except we only need USE
@@ -10,6 +10,7 @@ $environment{$_}={} for qw(USE); # INCREMENTALS, except we only need USE
 our %packages;
 our @profiles;
 our %use_masked_flags;
+our %use_defaults_flags;
 our %make_defaults_flags;
 our %default_flags;
 our %make_conf_flags;
@@ -28,31 +29,27 @@ sub read_make_globals();
 sub read_packages();
 sub read_profiles();
 sub read_sh($);
+sub read_use_defaults();
 sub read_use_mask();
 
 read_packages;
 read_profiles;
 read_use_mask;
+read_use_defaults;
 read_make_globals;
 read_make_defaults;
 read_make_conf;
 read_archs;
+read_use_descs;
 
-my $lastorder;
-for(reverse split /:/, $environment{USE_ORDER} || "env:pkg:conf:defaults:pkginternal:env.d") {
-	if($_ eq 'defaults') {
-		merge %default_flags, %make_defaults_flags;
-		merge %all_flags, %make_defaults_flags;
-	} elsif($_ eq 'conf') {
-		merge %all_flags, %make_conf_flags;
-	} else {
-		next;
-	}
-	$lastorder = $_;
-}
-if($lastorder ne 'conf') {
-	die "Sorry, USE_ORDER without make.conf overriding global USE flags are not currently supported by ufed.\n";
-}
+%default_flags = %make_defaults_flags;
+merge %default_flags, %use_defaults_flags;
+
+%all_flags = %default_flags;
+merge %all_flags, %make_conf_flags;
+
+for(keys %use_masked_flags)
+{ delete $all_flags{$_} if $use_masked_flags{$_} and exists $all_flags{$_} }
 
 sub have_package($) {
 	my ($cp) = @_;
@@ -131,7 +128,6 @@ sub read_archs() {
 
 sub read_make_conf() {
 	my %env = read_sh "/etc/make.conf";
-	merge (%env, read_sh('/etc/portage/make.conf'));
 	merge %make_conf_flags, %{$env{USE}} if exists $env{USE};
 	@portagedirs = $environment{PORTDIR};
 	push @portagedirs, split ' ', $environment{PORTDIR_OVERLAY} if defined $environment{PORTDIR_OVERLAY};
@@ -145,7 +141,7 @@ sub read_make_defaults() {
 }
 
 sub read_make_globals() {
-	for my $dir(@profiles, '/usr/share/portage/config') {
+	for my $dir(@profiles, '/etc') {
 		read_sh "$dir/make.globals";
 	}
 }
@@ -160,7 +156,7 @@ sub read_packages() {
 			if(open my $provide, '<', "/var/db/pkg/$cat/$pkg/PROVIDE") {
 				if(open my $use, '<', "/var/db/pkg/$cat/$pkg/USE") {
 					# could be shortened, but make sure not to strip off part of the name
-					$pkg =~ s/-\d+(?:\.\d+)*\w?(?:_(?:alpha|beta|pre|rc|p)\d*)?(?:-r\d+)?$//;
+					s/-\d+(?:\.\d+)*\w?(?:_(?:alpha|beta|pre|rc|p)\d*)?(?:-r\d+)?$//;
 					$packages{"$cat/$pkg"} = 1;
 					local $/;
 					my @provide = split ' ', <$provide>;
@@ -200,22 +196,21 @@ sub read_packages() {
 
 sub read_profiles() {
 	$_ = readlink '/etc/make.profile';
-	$_ = readlink '/etc/portage/make.profile' if not defined $_;
-	die "/etc\{,/portage\}/make.profile is not a symlink\n" if not defined $_;
+	die "/etc/make.profile is not a symlink\n" if not defined $_;
 	@profiles = norm_path '/etc', $_;
-	for (my $i = -1; $i >= -@profiles; $i--) {
-		for(noncomments "$profiles[$i]/parent") {
-			splice @profiles, $i, 0, norm_path $profiles[$i], $_;
+	PARENT: {
+		for(noncomments "$profiles[0]/parent") {
+			unshift @profiles, norm_path $profiles[0], $_;
+			redo PARENT;
 		}
 	}
-	push @profiles, '/etc/portage/profile';
 }
 
 sub read_sh($) {
 	my $BLANK = qr{(?:[ \n\t]+|#.*)+};         # whitespace and comments
-	my $IDENT = qr{([^ \\\n\t'"{}=#]+)};       # identifiers
+	my $IDENT = qr{([^ \\\n\t'"{}=]+)};        # identifiers
 	my $ASSIG = qr{=};                         # assignment operator
-	my $UQVAL = qr{((?:[^ \\\n\t'"#]+|\\.)+)}s;# unquoted value
+	my $UQVAL = qr{((?:[^ \\\n\t'"]+|\\.)+)}s; # unquoted value
 	my $SQVAL = qr{'([^']*)'};                 # singlequoted value
 	my $DQVAL = qr{"((?:[^\\"]|\\.)*)"}s;      # doublequoted value
 
@@ -230,10 +225,8 @@ sub read_sh($) {
 				/\G$IDENT/gc or die;
 				my $name = $1;
 				/\G$BLANK/gc;
-				if($name ne 'source') {
 				/\G$ASSIG/gc or die;
 				/\G$BLANK/gc;
-				}
 				die if pos == length;
 				my $value = '';
 				for(;;) {
@@ -257,20 +250,7 @@ sub read_sh($) {
 						last
 					}
 				}
-				if($name eq 'source') {
-					open my $f, '<', $value or die;
-					my $pos = pos;
-					substr($_, pos, 0) = do {
-						local $/;
-						my $text = <$f>;
-						die if not defined $text;
-						$text;
-					};
-					pos = $pos;
-					close $f or die;
-				} else {
 				$env{$name} = $value;
-				}
 			}
 		};
 		die "Parse error in $fname\n" if $@;
@@ -280,19 +260,21 @@ sub read_sh($) {
 	return %env if wantarray;
 }
 
+sub read_use_defaults() {
+	for my $dir(@profiles) {
+		for(noncomments "$dir/use.defaults") {
+			my ($flag, @packages) = split;
+			for(@packages)
+			{ $use_defaults_flags{$flag} = 1 if have_package $_ }
+		}
+	}
+}
+
 sub read_use_mask() {
 	for my $dir(@profiles) {
 		for(noncomments "$dir/use.mask") {
 			my $off = s/^-//;
-			$use_masked_flags{$_} = { '' => !$off };
-		}
-		for(noncomments "$dir/package.use.mask") {
-			my($pkg, @flags) = split;
-			for(@flags) {
-				my $off = s/^-//;
-				$use_masked_flags{$_}{''} ||= 0;
-				$use_masked_flags{$_}{$pkg} = !$off;
-			}
+			$use_masked_flags{$_} = !$off;
 		}
 	}
 }
